%% Header
% Purpose:  Calculation and graphical representation of pareto 
%           optimal PID controller tuning.
% Author:   Axel Lødemel Holene
% E-mail:   axel.holene@gmail.com
% Date:     13. February 2013


%% Initialising
clear all
close all
clc

%% Globals
global G cNum

%% Simulation parameters

simTime = 200 ;                                 % Simulation time
step_init = 0 ;                          % Initial starting value
dist_init = 0 ;                       % Initial disturbance value

step_value = 1 ;                                % Input step size
dist_value = 1 ;                          % Disturbance step size

step_time = 10 ;                               % Step change time
dist_time = 10 ;                                % Disurbance time

%% Process and controller parameters
modelId = 1 ;
G       = model(modelId) ;  % Generating system transfer function
cNum    = 3 ;                               % Controller option
% C     = controller([1 0 0]) ;

% sim('simModel')

% plot(y.time, y.signals.values)

%% Parameters for finding IAE weights
parm.Ms_s = 1.59 ;                                  % Ms setpoint
parm.simTime = 200 ;                   % Response simulation time
parm.Jweights   = [1 0]  ;        % Setting cost function weights
parm.IAEweights = [1 1]' ;   % Setting IAE cost function weigting
ctrl_0     = [1 0.5 0.5] ;             % Initial [Kc tau_i tau_d]

% Help functions
lb = [0 0 0]' ;                              % Lower tuning bound
%Jfun   = @(ctrl)costFun(ctrl, Jweights, IAEweights, simTime) ;
Jfun   = @(ctrl)costFun(ctrl, parm) ;
cFun = @(ctrl)conFun(ctrl,parm);

    fprintf('Finding Optimal IAE Weights \n')
    fprintf('Case: %g\n',modelId)

% Optimizing for IAE0_y
opt = optimset('algorithm','sqp','Display','off',...
               'TolCon',1e-4); 
    % 'active-set', 'trust-region-reflective', 'interior-point',
    % 'interior-point-convex', 'levenberg-marquardt', 
    % 'trust-region-dogleg', 'lm-line-search', or 'sqp'.
[ctrl IAE0_y optFlag] = fmincon(Jfun,ctrl_0,[],[],[],[],lb, ...
                        [],cFun,opt);

    fprintf('------------------------------------------------\n')
    fprintf('Cost  \t Tuning  \t   \t      \t \n')
    fprintf('J     \t Kc \t I \t D \t Ms \t flag \n')
    fprintf('------------------------------------------------\n')
    fprintf('%0.2f \t %.2f \t %.2f \t %.2f \t %.2f \t %d \n',...
        IAE0_y,ctrl,parm.Ms_s,optFlag)

% Optimizing for IAE0_d
parm.Jweights = [0 1]  ;
%Jfun   = @(ctrl)costFun(ctrl, Jweights, IAEweights, simTime) ;
Jfun   = @(ctrl)costFun(ctrl, parm) ;
[ctrl IAE0_d optFlag] = fmincon(Jfun,ctrl,[],[],[],[],lb, ...
                        [],cFun,opt);
    fprintf('%0.2f \t %.2f \t %.2f \t %.2f \t %.2f \t %d \n',...
        IAE0_d,ctrl,Ms_s,optFlag)

%% Setting weights
parm.Jweights   = [0.5 0.5] ;             % Cost function weights
parm.IAEweights = [IAE0_y IAE0_d]' ;                % IAE weights
ctrl_0 = [1 1 1] ;

%% Optimizer options
opt = optimset('algorithm','sqp','Display','off',...
               'TolCon',1e-4); 
    % 'active-set', 'trust-region-reflective', 'interior-point',
    % 'interior-point-convex', 'levenberg-marquardt', 
    % 'trust-region-dogleg', 'lm-line-search', or 'sqp'.


%% Gradient free optimization


% % Help function
% Jfun = @(ctrl)costFun(ctrl, Jweights, IAEweights, simTime) ;

% %% Generating pareto curve
% i = 1 ;                                       % Iteration counter
% Ms_span = 1.25:0.05:3 ;                           % Values for Ms
% JTune   = zeros(length(Ms_span),1) ;          % Init. cost vector
% IAETune = zeros(length(Ms_span),2) ;           % Init. IAE vector   Trenger jeg denne? Disse verdiene dukker jo opp i costFun, neppe enkle å hente frem...
% minTune = zeros(length(Ms_span),3) ;        % Init. tuning vector

%     fprintf('Generating the Pareto Optimal Curve\n')
%     fprintf('------------------------------------------------\n')
%     fprintf('Number of iterations: %d\n', length(Ms_span))
%     fprintf('J \t minTune \t \t Ms \t flag \t iterations left \n')
%     fprintf('------------------------------------------------\n')

% for Ms_s = Ms_span
%     cFun = @(ctrl)conFun(ctrl,Ms_s);     % Updating constraints
%     [ctrl J optFlag] = fmincon(Jfun,ctrl_0,[],[],[],[],lb, ...
%                        [],cFun,opt);               % Optimizing
%     minTune(i,:) = ctrl ;             % Storing controller tuning
%     JTune(i,:)   = J    ;           % storing cost function value

%     fprintf('%0.4e \t %.2f \t %.2f \t %.2f \t %.2f \t %d \t %d',...
%         J,ctrl,Ms_s,optFlag,length(Ms_span)-i)
%     fprintf('\n')

%     ctrl_0 = ctrl ;      % Start next search at last finish point
%     i = i + 1 ;                            % Incrementing counter
% end






