%% Header
% Purpose:  Producing controller transfer function from a set of
%           controller parameters.
% Author:   Axel Lødemel Holene
% E-mail:   axel.holene@gmail.com
% Date:     13. February 2013
% About:    

function C = controller2(varargin)
    % global cNum
    s = zpk('s') ;

    if nargin == 1
        parm = varargin ;
    elseif nargin == 2
        ctrl = varargin(1) ;
        parm = varargin(2) ;
    else
        fprintf('Too many controller arguments. \n')
    end
    
    G = parm.G ;
    cType = parm.cType ;
    ctrlr = parm.ctrlr ;
    cNum  = parm.cNum ;

    switch cType
        case 1 % PO
            switch ctrlr
                case 1 % PID
                    C.Kc = ctrl(1); C.I = ctrl(2); C.D = ctrl(3);
                    switch cNum
                        case 1  % Chriss': C=Kc((1-I-D)+I/s+Ds)
                            C.tf = C.Kc*((1 - C.I - C.D) + ...
                                   C.I/s + C.D*s) ; 
                        case 2  % Parallel: C=Kc(1+1/I/s+D*s)
                            C.tf = C.Kc*(1 + 1/C.I/s + C.D*s) ;
                        case 3  % Alt. parallel: C=Kc(1+I/s+D*s)
                            C.tf = C.Kc*(1 + C.I/s + C.D*s) ;
                        case 4 % Cascade controller 
                            C.tf = C.Kc*(C.tau_i*s + 1)/...
                            C.tau_i/s*(C.tau_d*s + 1) ;
                    end
                case 2 % PI
                    C.Kc = ctrl(1); C.I = ctrl(2);
                    switch cNum
                        case 1  % Chriss
                            C.tf = C.Kc*((1 - C.I) + C.I/s) ; 
                        case 2  % Parallel
                            C.tf = C.Kc*(1 + 1/C.I/s) ;
                        case 3  % AltParallel
                            C.tf = C.Kc*(1 + C.I/s) ;
                        case 4  % Cascade
                            C.tf = C.Kc*(1 + 1/C.I/s) ;
                    end
            end
        case 2 % SIMC
                C.tau_c = parm.tau_c ;% Closed-loop time constant
                C.Kc    = (1/G.k)*(G.tau1/(C.tau_c+G.theta)) ;
                C.tau_i = min(G.tau1, 4*(C.tau_c+G.theta)) ;
            switch ctrlr
                case 1  % PID
                    C.tau_d = parm.G.tau2 ;
                case 2  % PI
                    continue
            end
        case 3 % SMITH
            switch ctrlr
                case 1 % PID
                    C.Kc = ctrl(1); C.I = ctrl(2); C.D = ctrl(3);
                    switch cNum
                        case 1  % Chriss': C = Kc((1 - I - D) + I/s + Ds)
                            C.tf = C.Kc*((1 - C.I - C.D) + ...
                                   C.I/s + C.D*s) ; 
                        case 2  % Parallel: C = Kc(1 + 1/I/s + D*s)
                            C.tf = C.Kc*(1 + 1/C.I/s + C.D*s) ;
                        case 3  % Alt. parallel: C = Kc(1 + I/s + D*s)
                            C.tf = C.Kc*(1 + C.I/s + C.D*s) ;
                        case 4 % Cascade controller 
                            C.tf = C.Kc*(C.tau_i*s + 1)/...
                            C.tau_i/s*(C.tau_d*s + 1) ;
                    end
                case 2 % PI
                    switch cNum
                        case 1  % Chriss
                            C.tf = C.Kc*((1 - C.I) + C.I/s) ; 
                        case 2  % Parallel
                            C.tf = C.Kc*(1 + 1/C.I/s) ;
                        case 3  % AltParallel
                            C.tf = C.Kc*(1 + C.I/s) ;
                        case 4  % Cascade
                            C.tf = C.Kc*(1 + 1/C.I/s) ;
                    end
            end
            Csp.tf = C.tf/(1 + C.tf*G.tfnd*...
                     (1 - exp(-G.theta*s))) ;
    end
end