%% Header
% Purpose:	Cost function
% Author:   Axel Lødemel Holene (based on work done by Martin S.
%           Foss (2012)). 
% E-mail:	axel.holene@gmail.com
% Date:     14. February 2013
% About:    J = [Jw_y Jw_d]*[IAE_y IAE_d]./[IAEw_y IAEw_d]
%           ctrl =           struct of controller starting points
%                   Kc                            Controller gain
%                   tau_i                         Integral action
%                   tau_d                       Derivative action
%           Jweights   ([Jw_y Jw_d])        Cost function weights
%           IAEweights ([IAEw_y IAEw_d])            IAE weighting

function J = costFun(ctrl, parm)

    G  = parm.G ;
    C  = controller(ctrl) ;         % Generating controller
    Gy = 1/(1+G.tf*C.tf) ;         % Input feedback loop tf(y->e)
    Gd = -G.tf/(1+G.tf*C.tf) ;     % Dist. feedback loop tf(d->e)
    %% Output response
    [e t] = step([Gy; Gd]) ;

    %% Cost calculations
    IAE_0 = trapz(t, abs(e))';
    J     = parm.Jweights*(IAE_0./parm.IAEweights) ;
end