%% Header
% Purpose:  Creating the process transfer function G (struct).
% Author:   Axel Lødemel Holene (based on work done by Martin
%           Foss (2012)).
% E-mail:   axel.holene@gmail.com
% Date:     13. February 2013
% About:    G = 
%                   k                                Process gain
%                   tau1                    Process time constant
%                   tau2                    Process time constant
%                   theta                      Process time delay
%                   tf                  Process transfer function

function G = model(modelId)
    s = zpk('s') ;                   % Transfer function variable

    G.k     = 1   ;                         % Common process gain
    G.tau1  = 1   ;              % Common process time constant 1

    switch modelId
        case 1
            G.tau2  = 0.5*G.tau1 ;      % Process time constant 2
            G.theta = 1          ;           % Process time delay
        case 2
            G.tau2  = 0.8*G.tau1 ;      % Process time constant 2
            G.theta = 1 ;                    % Process time delay
        case 3
            G.tau2  = 0.3*G.tau1 ;      % Process time constant 2
            G.theta = 1 ;                    % Process time delay
        case 4
            G.tau2  = 0.5*G.tau1 ;      % Process time constant 2
            G.theta = G.tau2/1.5 ;           % Process time delay
        case 5
            G.tau2  = 0.8*G.tau1 ;      % Process time constant 2
            G.theta = G.tau2/1.5 ;           % Process time delay
        case 6
            G.tau2  = 0.3*G.tau1 ;      % Process time constant 2
            G.theta = G.tau2/1.5 ;           % Process time delay
        case 7
            G.tau2  = 0.5*G.tau1 ;      % Process time constant 2
            G.theta = G.tau2/2.0 ;           % Process time delay
        case 8
            G.tau2  = 0.8*G.tau1 ;      % Process time constant 2
            G.theta = G.tau2/2.0 ;           % Process time delay
        case 9
            G.tau2  = 0.3*G.tau1 ;      % Process time constant 2
            G.theta = G.tau2/2.0 ;           % Process time delay
        case 10
            G.tau1  = 0 ;
            G.tau2  = 0 ;
            G.theta = 1 ;
            G.tfnd  = G.k/s ;
            G.tf    = G.tfnd*exp(-G.theta*s) ;
            return
        case 11
            G.tau1  = 1 ;
            G.tau2  = 0 ;
            G.theta = 1 ;
        case 12
            G.tau1  = 8 ;
            G.tau2  = 0 ;
            G.theta = 1 ;
        case 13
            G.tau1  = 0 ;
            G.tau2  = 0 ;
            G.theta = 1 ;
    end
    %% Process transfer functions
    G.tfnd = G.k/((G.tau1*s + 1)*(G.tau2*s + 1)) ;
    G.tf   = G.tfnd*exp(-G.theta*s) ;
end