%% Header
% Purpose:  Finding SIMC controller settings. Accepting tau_c and
%           controller type as input. 
% Author:   Axel Lodemel Holene
% E-mail:   axel.holene@gmail.com
% Date:     13. February 2013
% About:    tau_c: closed loop time constant
%           G    : model for controller

function C = simcController(tau_c, G)
    s = zpk('s') ;                   % Transfer function variable
        %% Series controller
        C.tau_c = tau_c ;            % Closed-loop time constant
        C.Kc    = (1/G.k)*(G.tau1/(C.tau_c+G.theta)) ;         % Gain
        C.tau_i = min(G.tau1, 4*(C.tau_c+G.theta)) ;  % Integral time 
        try
            C.tau_d = G.tau2;     % Derivative action if 2. order
        catch
            C.tf = C.Kc*(1 + 1/C.tau_i/s) ;
            return
        end
        %% Controller transfer function 
        C.tf = C.Kc*(1 + 1/C.tau_i/s)*(C.tau_d*s + 1) ;
end