%% Header
% Purpose:  Calculation and graphical representation of SIMC 
%           tuning performance
% Author:   Axel Lodemel Holene
% E-mail:   axel.holene@gmail.com
% Date:     13. February 2013
% About:    ctrl = [Kc I D]

%% Initialising
clear all
close all
clc

% Adding commonFiles to search path
main   = fullfile('~/Dropbox/Documents/NTNU/Master_thesis/matlab/pareto') ;
shared = fullfile(main,'commonFiles') ;
addpath(shared)

casc = 'pid/cascade/POcasc_' ;
simc = 'pid/simc/POsimc_'    ;
pid  = 'pid/chriss/POpid_'   ;
pi   = 'pi/POpi_'            ;

modelName = {'case1', 'case2', 'case3', 'case4', 'case5', ...
             'case6', 'case7', 'case8', 'case9'} ;
ctrlRule  = {casc, simc, pid, pi} ;        % Locations of results
ctrls = {'POcasc', 'POsimc', 'POpid', 'POpi'} ; % Name of results
modelMax = 9 ;
MsSet   = [1.25, 1.6, 2.4] ;           % Set of Ms values

% for ctrlId = 1:length(ctrls)
for ctrlId = 1:1
    for modelId = 1:modelMax ;
        % Loading data file
        load(fullfile(main, 'results', [ctrlRule{ctrlId}, ...
                      modelName{modelId},'.mat'])) ;
        % Setting main struct
        this = getfield(res,ctrls{ctrlId}) ;
        tau_1 = res.G.tau1  ;
        tau_2 = res.G.tau2  ;
        theta = res.G.theta ;
        if modelId == 1
            tune = zeros([size(this.tuning),modelMax]) ;
            Ms   = zeros([length(this.Ms),modelMax]) ;
            ctrl = zeros(length(MsSet), size(tune,2), modelMax) ;
            tau  = zeros(modelMax,1) ;
            delay= zeros(modelMax,1) 
        end
        % Loading model parameters
        tune(:,:,modelId) = this.tuning ;
        Ms(1:length(this.Ms),modelId)       = this.Ms ;
        tau(modelId) = tau_2/tau_1 ;
        delay(modelId) = theta 
        % Finding values
        i = 1 ;
        for Ms_s = MsSet
            n = find(Ms_s == Ms(:,modelId)) ;
            ctrl(i,:,modelId) = tune(n,:,modelId) ; 
            i = i + 1 ;
        end    
    end
    % Plotting routine
    % subplot(2,2,ctrlId)
    hold on
    i = 1 ;
    for i = 1:length(MsSet)
        j = 1 ;
        for j = 1:size(ctrl,2)
            k = 1 ;
            vec = [] ;
            for k = 1:length(tau)
                vec = [vec, ctrl(i, j, k)] ;
                k = k + 1 ;
            end
            plot(tau, vec)
            j = j + 1 ;
        end
        i = i + 1 ;
    end
    hold off
end                                             % Ending for-loop

% % Removing zeros
% % J(J==0)     = NaN ;
% % Ms(Ms == 0) = NaN ;
% % Mt(Mt == 0) = NaN ;

% % Plotting routine
% figure(ctrlId)
% clf
% subplot(2,1,1), plot(Ms, J, 'LineWidth', 1) ;
% axis([0 5 0 4])
% xlabel('Robustness, $M_s$', 'interpreter','latex','FontSize',12)
% ylabel('Performance, $J$', 'interpreter','latex','FontSize',12)
% titleName = {'Case 1', 'Case 2', 'Case 3', 'Case 4', ...
%              'Case 5', 'Case 6', 'Case 7', 'Case 8', ...
%              'Case 9'} ;
% title(titleName{modelId},'interpreter','latex','FontSize',12)
% legend('Cascade-pid','SIMC-pid','PO-pid','PO-pi','Location','NorthEast')

% subplot(2,1,2), plot(Mt, Ms)
% ylabel('$M_s$', 'interpreter', 'latex', 'FontSize', 12)
% xlabel('$M_t$', 'interpreter', 'latex', 'FontSize', 12)
% title(titleName{modelId},'interpreter','latex','FontSize',12)
% legend('Cascade-pid','SIMC-pid','PO-pid','PO-pi','Location','NorthEast')

% save([main,'/results/merged/','lag2tuning',...
%     num2str(modelName{modelId}),'.mat'],'res')
% saveas(gcf,['figures/merged/','lag2tuning'...
%        num2str(modelName{modelId}),'.fig'])

restoredefaultpath