%% Header
% Purpose:  Calculation and graphical representation of SIMC 
%           tuning performance
% Author:   Axel Lodemel Holene
% E-mail:   axel.holene@gmail.com
% Date:     13. February 2013
% About:    ctrl = [Kc I D]

%% Initialising
clear all
close all
clc

% Adding commonFiles to search path
main   = fullfile('~/Dropbox/Documents/NTNU/Master_thesis/matlab/pareto') ;
shared = fullfile(main,'commonFiles') ;
addpath(shared)

casc = 'pid/cascade/POcasc_' ;
simc = 'pid/simc/POsimc_'    ;
pid  = 'pid/chriss/POpid_'   ;
pi   = 'pi/POpi_'            ;

modelName = {'case1', 'case2', 'case3', 'case4', 'case5', ...
             'case6', 'case7', 'case8', 'case9'} ;
ctrlRule  = {casc, simc, pid, pi} ;        % Locations of results
ctrls = {'POcasc', 'POsimc', 'POpid', 'POpi'} ; % Name of results

for modelId = 1:1 ;
MsSet   = 1.59 ;        % Setting Ms value for C = f(tau_2/tau_1)

for ctrlId = 1:length(ctrls)
    % Loading data file
    load(fullfile(main, 'results', [ctrlRule{ctrlId}, ...
                  modelName{modelId},'.mat'])) ;
    
    % Setting main struct
    this = getfield(res,ctrls{ctrlId}) ;

    % Initializing matrices if needed
    if ctrlId == 1
        J     = zeros(length(this.Jtune),length(ctrls)) ;
        Ms    = zeros(length(this.Ms),   length(ctrls)) ;
        Mt    = zeros(length(this.Mt),   length(ctrls)) ;
        k     = res.G.k     ;
        tau_1 = res.G.tau1  ;
        tau_2 = res.G.tau2  ;
        theta = res.G.theta ;
        h = zeros(9,1) ;                         % Figure handles
        % lcell = cell(1,length(ctrls)) ;        % Legend data cell
    end
    % Loading model parameters
    J(1:length(this.Jtune),ctrlId) = this.Jtune  ;
    Ms(1:length(this.Ms),ctrlId)   = this.Ms     ;
    Mt(1:length(this.Mt),ctrlId)   = this.Mt     ;
end

% Removing zeros
J(J==0)     = NaN ;
Ms(Ms == 0) = NaN ;
Mt(Mt == 0) = NaN ;

% Plotting routine
figure(modelId)
clf
subplot(2,1,1), plot(Ms, J, 'LineWidth', 1) ;
axis([0 5 0 4])
xlabel('Robustness, $M_s$', 'interpreter','latex','FontSize',12)
ylabel('Performance, $J$', 'interpreter','latex','FontSize',12)
titleName = {'Case 1', 'Case 2', 'Case 3', 'Case 4', ...
             'Case 5', 'Case 6', 'Case 7', 'Case 8', ...
             'Case 9'} ;
title(titleName{modelId},'interpreter','latex','FontSize',12)
legend('Cascade-pid','SIMC-pid','PO-pid','PO-pi','Location','NorthEast')

subplot(2,1,2), plot(Mt, Ms)
ylabel('$M_s$', 'interpreter', 'latex', 'FontSize', 12)
xlabel('$M_t$', 'interpreter', 'latex', 'FontSize', 12)
title(titleName{modelId},'interpreter','latex','FontSize',12)
legend('Cascade-pid','SIMC-pid','PO-pid','PO-pi','Location','NorthEast')

save([main,'/results/merged/',...
    num2str(modelName{modelId}),'.mat'],'res')
saveas(gcf,['figures/merged/',...
       num2str(modelName{modelId}),'.fig'])

end                                             % Ending for-loop
restoredefaultpath

%s{k} = sprintf('Plot of x.^%d', k);
% cascade: resPOpid_case1
% nograd: cascade_case1noGrad
% pi: resPOpi_case1
% pid: resPOpid_case1
% simc: resPOpidsimc_case1

