%% Header
% Purpose:  Calculation and graphical representation of pareto 
%           optimal PID controller tuning.
% Author:   Axel Lodemel Holene
% E-mail:   axel.holene@gmail.com
% Date:     13. February 2013
% About:    ctrl = [Kc I D]


%% Initialising
clear all
close all
clc

% Adding commonFiles to search path
main   = fullfile('~/Dropbox/Documents/NTNU/Master_thesis/matlab/pareto') ;
shared = fullfile(main,'commonFiles') ;
addpath(shared)

%% Globals
global G cNum

startTime = cputime ;              % Start total computation time

%% Simulation, process and controller  parameters
modelName = {'case10', 'case11', 'case12', 'case13'} ;
ctrlType  = {'pi', 'pid_par', 'pid_casc'} ;

for modelId = 1:4 ;                                    % Model ID
cNum    = 1 ;                                 % Controller option
simTime = 100 ;                                 % Simulation time
G       = smithModel(modelId) ;           % Generating process TF

parm.simTime = simTime ;

load([main,'/results/pi/', 'POpi_',...
num2str(modelName{modelId}),'.mat']) ;
% parm.ctrls = res.POpi.ctrls ;
parm.ctrls = res.POpi.tuning ;
IAE0_y = res.IAEweights(1) ;
IAE0_d = res.IAEweights(2) ;
parm.IAEweights = [IAE0_y IAE0_d]' ;                % IAE weights
parm.Jweights   = [0.5 0.5] ;             % Cost function weights
fprintf('Optimal %s-controls loaded\n', ctrlType{cNum})

% [num den] = tfdata(parm.ctrls);

% Csp{length(den)} = [] ;                     % Smith predictor TFs
% J       = zeros(length(den),1) ;
% Ms_span = zeros(length(den),1) ;

% Csp{length(parm.ctrls)} = [] ;
J = zeros(size(parm.ctrls,1),1) ;
Ms = zeros(size(parm.ctrls,1),1) ;

i = 1 ;
for i = 1:size(parm.ctrls,1)
    C = controller(parm.ctrls(i,:)) ; % Generating PI TF
    Ct = smithController(C.tf) ;
    J(i) = smithCost(Ct, parm) ;
    Ms(i) = MsSmith(Ct) ;
    % Csp{i} = smithController(C) ;
    % J = smithCost(Csp{i},parm) ;
    % Ms_span = MsSmith(Csp{i}) ;
    fprintf('Computation #%d done.\n', i)
    i = i + 1;
end

%% Ploting routine
figure(modelId)                 % Figure number from model number
hold on
plot(Ms, J, res.POpi.Ms, res.POpi.Jtune) ;% Plotting Ms and cost values
axis([0 3 0 6])                                 % Adjusting axis'
xlabel('Robustness, $M_s$', 'interpreter','latex','FontSize',12)
ylabel('Performance, $J$', 'interpreter','latex','FontSize',12)
titleName = {'Case 1', 'Case 2',  'Case 3',  'Case 4'} ;
title(titleName{modelId},'interpreter','latex','FontSize',12)
legend('PI Smith', 'PO PI','Location', 'NorthEast')
hold off



end






